from jsonschema import validate
from jsonschema.exceptions import ValidationError
from rest_framework import serializers
from .models import (
    Guild,
    Player,
    Game,
    PlayerGuild,
    PlayerGameData,
    GuildGameDataSchema,
    WowClasses,
)


class GuildSerializer(serializers.ModelSerializer):
    class Meta:
        model = Guild
        fields = ["id", "name", "tag", "creation_date"]
        read_only_fields = ["id", "creation_date"]


class PlayerSerializer(serializers.ModelSerializer):
    class Meta:
        model = Player
        fields = ["id", "name", "creation_date"]
        read_only_fields = ["id", "creation_date"]


class GameSerializer(serializers.ModelSerializer):
    class Meta:
        model = Game
        fields = ["id", "name", "creation_date"]
        read_only_fields = ["id", "creation_date"]


class PlayerGameDataSerializer(serializers.ModelSerializer):
    class Meta:
        model = PlayerGameData
        fields = ["id", "player", "game", "creation_date", "data"]
        read_only_fields = ["id", "creation_date"]

    def validate_data(self, value):
        game_id = self.initial_data["game"]
        try:
            schema_obj = GuildGameDataSchema.objects.get(
                game=game_id, guild=self.initial_data["guild"]
            )
            schema = schema_obj.schema

            validate(value, schema)

        except GuildGameDataSchema.DoesNotExist:
            raise serializers.ValidationError(
                "Schema does not exist for this guild and game"
            )
        except ValidationError as e:
            raise serializers.ValidationError(e.message)

        return value


class GuildGameDataSchemaSerializer(serializers.ModelSerializer):
    class Meta:
        model = GuildGameDataSchema
        fields = ["id", "guild", "game", "schema"]
        read_only_fields = ["id"]


class PlayerGuildSerializer(serializers.ModelSerializer):
    class Meta:
        model = PlayerGuild
        fields = ["id", "player", "guild", "creation_date"]
        read_only_fields = ["id", "creation_date"]


class WowClassesSerializer(serializers.ModelSerializer):
    class Meta:
        model = WowClasses
        fields = ["id", "name", "creation_date"]
        read_only_fields = ["id", "creation_date"]
